@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,25 +35,33 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-162.js';
+var gTestfile = 'regress-385393-02.js';
+
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 385393;
+var summary = 'Regression test for bug 385393';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
-//-------  Comment #162  From  Gary Kwong
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+function test()
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
+ 
+  try
+  {
+    (4).__lookupGetter__("w");
+  }
+  catch(ex)
+  {
+  }
 
-// Assertion failure: !OBJ_GET_CLASS(cx, proto)->getObjectOps, at ../jsobj.cpp:2030
+  reportCompare(expect, actual, summary);
 
-jit(true);
-__defineGetter__("x3", Function);
-undefined = x3;
-undefined.prototype = [];
-for (var z = 0; z < 4; ++z) { new undefined() }
-jit(false);
-
-reportCompare(expect, actual, summary);
+  exitFunc ('test');
+}
