@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): nanto_vi (TOYAMA Nao)
+ * Contributor(s): Brendan Eich 
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,13 +35,14 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-433279-01.js';
+var gTestfile = 'regress-452498-191.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 433279;
-var summary = 'Do not assert: pn != tc->parseContext->nodeList';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
+var actual = '';
+var expect = '';
 
+//-------  Comment #191  From  Brendan Eich
 
 //-----------------------------------------------------------------------------
 test();
@@ -52,9 +53,27 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  var { sin, PI } = Math; sin(PI / 2);
 
+  expect = actual = 'No Error';
+  try
+  {
+    eval('{ var x; {let x;} }');
+  }
+  catch(ex)
+  {
+    actual = ex + '';
+  }
+  reportCompare(expect, actual, summary);
+
+  expect = 'TypeError: redeclaration of let x';
+  try
+  {
+    eval('{ let x; {var x;} }');
+  }
+  catch(ex)
+  {
+    actual = ex + '';
+  }
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
