@@ -70,6 +70,7 @@ pkix_ComCRLSelParams_Destroy(
         PKIX_DECREF(params->date);
         PKIX_DECREF(params->maxCRLNumber);
         PKIX_DECREF(params->minCRLNumber);
+        PKIX_DECREF(params->crldpList);
 
 cleanup:
 
@@ -405,6 +406,9 @@ pkix_ComCRLSelParams_Duplicate(
         PKIX_DUPLICATE(old->cert, &new->cert, plContext,
                     PKIX_OBJECTDUPLICATECERTFAILED);
 
+        PKIX_DUPLICATE(old->crldpList, &new->crldpList, plContext,
+                    PKIX_OBJECTDUPLICATECERTFAILED);
+
         PKIX_DUPLICATE(old->issuerNames, &new->issuerNames, plContext,
                     PKIX_OBJECTDUPLICATEISSUERNAMESFAILED);
 
@@ -488,6 +492,7 @@ PKIX_ComCRLSelParams_Create(
         /* initialize fields */
         params->issuerNames = NULL;
         params->cert = NULL;
+        params->crldpList = NULL;
         params->date = NULL;
         params->nistPolicyEnabled = PKIX_TRUE;
         params->maxCRLNumber = NULL;
@@ -831,3 +836,24 @@ cleanup:
 
         PKIX_RETURN(COMCRLSELPARAMS);
 }
+
+
+PKIX_Error*
+PKIX_ComCRLSelParams_SetCrlDp(
+         PKIX_ComCRLSelParams *params,
+         PKIX_List *crldpList,
+         void *plContext)
+{
+    PKIX_ENTER(COMCRLSELPARAMS, "PKIX_ComCRLSelParams_SetCrlDp");
+    PKIX_NULLCHECK_ONE(params); /* minCRLNumber can be NULL - from spec */
+
+    PKIX_INCREF(crldpList);
+    params->crldpList = crldpList;
+
+    PKIX_CHECK(PKIX_PL_Object_InvalidateCache
+               ((PKIX_PL_Object *)params, plContext),
+               PKIX_OBJECTINVALIDATECACHEFAILED);
+cleanup:
+
+        PKIX_RETURN(COMCRLSELPARAMS);
+}
